package org.netkernel.http.transport;

import org.mortbay.jetty.Connector;
import org.mortbay.jetty.Handler;
import org.mortbay.jetty.Server;
import org.mortbay.log.Logger;
import org.mortbay.xml.XmlConfiguration;
import org.netkernel.container.ILogger;
import org.netkernel.container.config.IConfiguration;
import org.netkernel.layer0.meta.IEndpointStateMeta;
import org.netkernel.layer0.nkf.INKFLocale;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.representation.IReadableBinaryStreamRepresentation;
import org.netkernel.layer0.representation.impl.HDSBuilder;
import org.netkernel.layer0.util.Utils;
import org.netkernel.module.standard.endpoint.StandardTransportImpl;
import org.netkernel.request.IRequestResponseFields;
import org.netkernel.request.impl.RequestResponseFieldsImpl;


public class HTTPTransportEndpoint extends StandardTransportImpl implements IEndpointStateMeta
{
	private Server mServer;
	private static String sServerString;
	private INetKernelHandler mHandler;
	
	public HTTPTransportEndpoint()
	{	declareSupportedVerbs(0);
		System.setProperty("JETTY_NO_SHUTDOWN_HOOK", Boolean.TRUE.toString());
	}
			
	@Override
	protected void postCommission(INKFRequestContext helper) throws Exception
	{	helper.logFormatted(INKFLocale.LEVEL_INFO,"MSG_COMMISSION_TRANSPORT",null);
		IConfiguration kcfg=helper.getKernelContext().getKernel().getConfiguration();
		String product=kcfg.getString("netkernel.instance.product", "NetKernel");
		String major = kcfg.getString("netkernel.instance.version.major", "?");
		//String minor =kcfg.getString("netkernel.instance.version.minor", "?");
		sServerString=helper.formatMessage("SERVER_STRING", product+" "+major);
		try
		{	IReadableBinaryStreamRepresentation serverConfig=(IReadableBinaryStreamRepresentation)helper.source("param:config",IReadableBinaryStreamRepresentation.class);
			final ILogger nkLogger = helper.getKernelContext().getKernel().getLogger();
			Logger l=new Logger()
			{
				
				public void debug(String arg0, Throwable arg1)
				{	nkLogger.logRaw(ILogger.DEBUG, HTTPTransportEndpoint.this, arg0);
					nkLogger.logRaw(ILogger.DEBUG, HTTPTransportEndpoint.this, Utils.throwableToString(arg1));
				}

				public void debug(String arg0, Object arg1, Object arg2)
				{	nkLogger.logRaw(ILogger.DEBUG, HTTPTransportEndpoint.this, arg0);
				}

				public Logger getLogger(String arg0)
				{	return this;
				}

				public void info(String arg0, Object arg1, Object arg2)
				{	nkLogger.logRaw(ILogger.INFO, HTTPTransportEndpoint.this, arg0);
				}

				public boolean isDebugEnabled()
				{	return nkLogger.shouldLog(ILogger.DEBUG, HTTPTransportEndpoint.this);
				}

				public void setDebugEnabled(boolean arg0)
				{	
				}

				public void warn(String arg0, Throwable arg1)
				{	nkLogger.logRaw(ILogger.WARNING, HTTPTransportEndpoint.this, arg0);
					nkLogger.logRaw(ILogger.WARNING, HTTPTransportEndpoint.this, Utils.throwableToString(arg1));
				}

				public void warn(String arg0, Object arg1, Object arg2)
				{	nkLogger.logRaw(ILogger.WARNING, HTTPTransportEndpoint.this, arg0);
				}
			};
			org.mortbay.log.Log.setLog(l);
		    mServer = new Server();
		    XmlConfiguration configuration = new XmlConfiguration(serverConfig.getInputStream()); 
		    configuration.configure(mServer);
		    
		    //Find NetKernelHandler and register helper - have to do extra work as server behaves differently between 1 and many handlers.
		    INetKernelHandler nkHandler=null;
		    Handler[] handlers=mServer.getHandlers();
		    if(handlers!=null)
		    {   //many handlers
		    	for(Handler h: handlers)
			    {	if(testNKHandler(h))
			    	{	nkHandler=(INetKernelHandler)h;
			    		break;
			    	}
			    }
		    }
		    else
		    {	//only one handler
		    	Handler h=mServer.getHandler();
		    	if(h!=null)
		    	{	if(testNKHandler(h))
		    		{	nkHandler=(INetKernelHandler)h;
		    		}
		    	}
		    }
		    if(nkHandler==null)
		    {	throw helper.createFormattedException("EX_NO_NETKERNEL_HANDLER","MSG_NO_NETKERNEL_HANDLER",null,null);		    	
		    }
		    nkHandler.registerContext(getTransportContext());
		    mHandler=nkHandler;
		    
		    for(Connector i: mServer.getConnectors())
		    {    helper.logFormatted(INKFLocale.LEVEL_INFO,"MSG_START_TRANSPORT", new Object[] {"HTTP", i.getPort()  });
		    }
		    mServer.setStopAtShutdown(false);
		    mServer.start();
		    
		    //Initialize stats.
		    for(Connector c: mServer.getConnectors())
		    {	if(c.getStatsOn())
		    	{	c.statsReset();
		    	}
		    }

		}
		catch(Exception e)
		{	helper.logFormatted(INKFLocale.LEVEL_WARNING,"EX_START_FAIL", null);
			e.printStackTrace();
			throw e;
		}
	}

	@Override
	protected void preDecommission(INKFRequestContext helper) throws Exception
	{	helper.logFormatted(INKFLocale.LEVEL_INFO,"MSG_DECOMMISSION_TRANSPORT",null);
		if(mServer!=null)
		{	mServer.setGracefulShutdown(100);
			mServer.stop();
			Thread.sleep(200);
			mServer.destroy();
			mHandler.destroy();
			mHandler=null;
		}
		mContext=null;
	}
	
	private boolean testNKHandler(Handler h)
	{	return h.toString().startsWith("NetKernelHandler");
	}
	
	static String getServerHeader()
	{	return sServerString;
	}

	public IRequestResponseFields getState()
	{	RequestResponseFieldsImpl fields=new RequestResponseFieldsImpl(super.getState());
		fields.put("serverstring", sServerString);
		fields.put("handler", mHandler.getClass().getName());
		fields.put("requests", mHandler.requestCount());
		int cid=0;
		for(Connector c: mServer.getConnectors())
		{	String conid="con"+cid+":";
			fields.put(conid+"connector", c.getClass().getName());
			fields.put(conid+"port", c.getPort());
			if(c.getStatsOn())
			{
				fields.put(conid+"Requests", c.getRequests());
				fields.put(conid+"Connections", c.getConnections());
				fields.put(conid+"ConnectionsOpen", c.getConnectionsOpen());
				fields.put(conid+"ConnectionsOpenMax", c.getConnectionsOpenMax());
				fields.put(conid+"ConnectionsRequestsAve", c.getConnectionsRequestsAve());
				fields.put(conid+"ConnectionsRequestsMax", c.getConnectionsRequestsMax());
				fields.put(conid+"ConnectionsDurationAve", c.getConnectionsDurationAve());
				fields.put(conid+"ConnectionsDurationMax", c.getConnectionsDurationMax());
			}
			else
			{	fields.put(conid+"StatsNotSetOn", 0);
			}
			cid++;
		}
		return fields;
	}

}
